/*	$Id: netshring.h 2007/07/05 19:10:49 mallet $ */

/* Copyright (c) 2007 Anthony Mallet                  --  Fri Jun 15 2007
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice, this list of conditions and the following disclaimer.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice,  this list of  conditions and the following disclaimer in
 *      the  documentation  and/or  other   materials provided  with  the
 *      distribution.
 *
 * THIS  SOFTWARE IS PROVIDED BY  THE  COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND  ANY  EXPRESS OR IMPLIED  WARRANTIES,  INCLUDING,  BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES  OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR  PURPOSE ARE DISCLAIMED. IN  NO EVENT SHALL THE COPYRIGHT
 * HOLDERS OR      CONTRIBUTORS  BE LIABLE FOR   ANY    DIRECT, INDIRECT,
 * INCIDENTAL,  SPECIAL,  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF  SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE   OF THIS SOFTWARE, EVEN   IF ADVISED OF   THE POSSIBILITY OF SUCH
 * DAMAGE.
 */
#ifndef H_NETSHRING
#define H_NETSHRING

#include <stdio.h>
#include <pthread.h>

typedef enum netshring_err {
  SHRNG_OK = 0,
  SHRNG_NOMEM,
  SHRNG_INVAL,
  SHRNG_LOCK,
  SHRNG_CONNECT,
} netshring_err_t;

typedef enum netshring_type {
  SHRNG_CLOSED,
  SHRNG_SOURCE,
  SHRNG_DRAIN
} netshring_type_t;

typedef enum netshring_opt {
  SHRNG_ASYNC	= 0,
  SHRNG_SYNC	= 0x1
} netshring_opt_t;

typedef struct netshring_id {
  netshring_type_t type;	/**< server/client side */

  int sockfd;			/**< server socket */
  int clients;			/**< number of connected clients */
  int *clientfd;		/**< array of clients sockets */

  pthread_mutex_t mutex;	/**< lock for this structure */
} *netshring_id_t;


/* --- API ----------------------------------------------------------- */

#ifdef __cplusplus
extern "C" {
#endif

  netshring_id_t	netshring_create(const char *name);
  void			netshring_delete(netshring_id_t shid);

  netshring_id_t	netshring_connect(const char *name,
				netshring_opt_t sync);
  void			netshring_disconnect(netshring_id_t shid);

  int			netshring_pop(netshring_id_t shid, void *buffer,
				      size_t max, netshring_opt_t sync);
  int			netshring_push(netshring_id_t shid, const void *buffer,
				size_t length);

#ifdef __cplusplus
}
#endif

#endif /* H_NETSHRING */
